#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <richedit.h>
#include "AkelDLL.h"

#ifndef GT_SELECTION
  #define GT_SELECTION 2
#endif

int SetLayout();
void RestoreLayout(int nLayout);
int GetSelTextA(HWND hWnd, char **pText);
int GetSelTextW(HWND hWnd, wchar_t **wpText);
void FreeSelText(LPVOID pText);


//Plugin extern function
extern "C" void __declspec(dllexport) GetSelTextTest(PLUGINDATA *pd)
{
  char *pResult;
  wchar_t *wpResult;
  int nLen;

  if (pd->bOldWindows)
  {
    if (nLen=GetSelTextA(pd->hWndEdit, &pResult))
    {
      MessageBox(pd->hMainWnd, pResult, "Test", MB_OK);

      FreeSelText(pResult);
    }
    else MessageBox(pd->hMainWnd, "Text doesn't selected", "Test", MB_OK);
  }
  else
  {
    if (nLen=GetSelTextW(pd->hWndEdit, &wpResult))
    {
      MessageBoxW(pd->hMainWnd, wpResult, L"Test", MB_OK);

      FreeSelText(wpResult);
    }
    else MessageBoxW(pd->hMainWnd, L"Text doesn't selected", L"Test", MB_OK);
  }
}

//Plugin extern function
extern "C" void __declspec(dllexport) OpenDialogTest(PLUGINDATA *pd)
{
  SendMessage(pd->hMainWnd, WM_COMMAND, IDM_FILE_OPEN, 0);
}


int SetLayout()
{
  LANGID wLangSystem=GetUserDefaultLangID();
  int nLayout=LOWORD(GetKeyboardLayout(0));

  if (nLayout != wLangSystem && !ActivateKeyboardLayout((HKL)wLangSystem, 0))
    nLayout=0;
  return nLayout;
}

void RestoreLayout(int nLayout)
{
  if (nLayout) ActivateKeyboardLayout((HKL)nLayout, 0);
}

int GetSelTextA(HWND hWnd, char **pText)
{
  CHARRANGE cr;
  int nLen;
  TEXTRANGE txtrng;
  int nLayout;

  SendMessage(hWnd, EM_EXGETSEL, 0, (LPARAM)&cr);

  if (cr.cpMin == cr.cpMax) goto Error;
  nLen=(cr.cpMax - cr.cpMin) + 1;

  if (*pText=(char *)HeapAlloc(GetProcessHeap(), 0, nLen + 1))
  {
    txtrng.chrg.cpMin=cr.cpMin;
    txtrng.chrg.cpMax=cr.cpMax;
    txtrng.lpstrText=*pText;

    nLayout=SetLayout();
    SendMessage(hWnd, EM_GETTEXTRANGE, 0, (LPARAM)&txtrng);
    RestoreLayout(nLayout);

    return nLen;
  }

  Error:
  *pText=NULL;
  return 0;
}

int GetSelTextW(HWND hWnd, wchar_t **wpText)
{
  CHARRANGE cr;
  GETTEXTEX gt;
  int nLen;
  int nLenW;

  SendMessage(hWnd, EM_EXGETSEL, 0, (LPARAM)&cr);

  if (cr.cpMin == cr.cpMax) goto Error;
  nLen=(cr.cpMax - cr.cpMin) + 1;
  nLenW=nLen * sizeof(wchar_t);

  if (*wpText=(wchar_t *)HeapAlloc(GetProcessHeap(), 0, nLenW + 2))
  {
    gt.cb=nLenW;
    gt.flags=GT_SELECTION;
    gt.codepage=1200;
    gt.lpDefaultChar=NULL;
    gt.lpUsedDefChar=NULL;
    SendMessage(hWnd, EM_GETTEXTEX, (WPARAM)&gt, (LPARAM)*wpText);

    return nLen;
  }

  Error:
  *wpText=NULL;
  return 0;
}

void FreeSelText(LPVOID pText)
{
  if (pText) HeapFree(GetProcessHeap(), 0, pText);
}

//Entry point
BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
{
  if (fdwReason == DLL_PROCESS_ATTACH)
  {
  }  
  else if (fdwReason == DLL_THREAD_ATTACH)
  {
  }
  else if (fdwReason == DLL_THREAD_DETACH)
  {
  }
  else if (fdwReason == DLL_PROCESS_DETACH)
  {
  }
  return TRUE;
}
